/*

Copyright (C) 2003 J.D. Cole <jd@transientresearch.com>

This file is part of Octave.

Octave is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Octave is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Octave; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

$Id: oct-pi.h,v 1.8.2.1 2003/03/03 22:23:58 jd Exp $
*/

#if !defined (octave_parallel_interface_h)
#define octave_parallel_interface_h 1

#if defined (__GNUG__) && defined (USE_PRAGMA_INTERFACE_IMPLEMENTATION)
#pragma interface
#endif

#include "dynamic-ld.h"
#include "oct-pil.h"

class octave_parallel_interface;

// !! should I use typedef here? They should at least be in the oct-pil.h
//    provided for the PIL builder.

// If TRUE, user wants to start parallel libraries
// (--parallel)
extern bool parallel_flag;

extern octave_parallel_interface parallel_interface;

class 
octave_parallel_interface
{
public:

  enum parallel_state
  {
    pre_initialize,
    started,
    stopped,
    failed_to_load,
    failed_dependencies,
    failed_to_initialize
  };

  octave_parallel_interface (void);

  ~octave_parallel_interface (void);

  // Octave hooks

  /// Library initialization and/or command line stripping
  pi_error initialize (int *argc, char **argv[]);

  /// Install builtin variables and/or functions associated with PIL
  pi_error install_builtins (void);

  pi_error finalize (void);

  std::string get_error_string (void) { return current_error_string; }

private:
  // The full path name of the Parallel Interface Library
  std::string pil_full_path_name;

  // Dynamical lib interface
  void *pil_handle;

  // Current error state
  pi_error current_error_state;
  std::string current_error_string;

  // Was initialization successful ?
  parallel_state state;
  
  // The following are non NULL if they were found. The bare
  // minimum is eval function (!! Is this right? A PIL could
  // just provide builtins and nothing else.)
  pil_initialize_fcn_type		initialize_fcn;
  pil_install_builtins_fcn_type	        install_builtins_fcn;
  pil_finalize_fcn_type		        finalize_fcn;

  pil_error_string_fcn_type             error_string_fcn;

  void set_pil_error_string(const std::string msg);
  pi_error open (const std::string pil_name);
  void *search (const std::string fcn_name);
  void close (void);
};

#endif

/*
;;; Local Variables: ***
;;; mode: C++ ***
;;; End: ***
*/
